import { initDb } from '.';
import uri from '../lib/ids';
import { getFamily } from './lib';
import { putNewRte } from './rte';
import { putNewRtept } from './rtept';
import { putNewTrk } from './trk';
import { putNewTrkpt } from './trkpt';
import { putNewTrkseg } from './trkseg';
import { putNewWpt } from './wpt';

declare global {
  var db: any;
}

const originalDateNow = globalThis.Date.now;

describe('getFamily', () => {
  beforeEach(async () => {
    await initDb({});
    globalThis.Date.now = () => 0;
  });
  afterEach(async () => {
    await db.destroy();
    db = undefined;
    globalThis.Date.now = originalDateNow;
  });
  test('returns two rows after a gpx and a track have been inserted.', async () => {
    await putNewTrk();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/08640000000000000",
      "key": "gpx/08640000000000000",
      "value": Object {
        "rev": "1-e8125b3d924c831968288bf8786e8651",
      },
    },
    Object {
      "id": "gpx/08640000000000000/2trk/000000",
      "key": "gpx/08640000000000000/2trk/000000",
      "value": Object {
        "rev": "1-a4d5a6dfa66766fd3e42c6ee328436f7",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });
  test('also returns the docs if required.', async () => {
    await putNewTrk();
    const allDocs: any = await getFamily('gpx/', {
      include_docs: true,
    });
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "doc": Object {
        "_id": "gpx/08640000000000000",
        "_rev": "1-e8125b3d924c831968288bf8786e8651",
        "doc": Object {
          "$": Object {
            "creator": "dyomedea version 0.000002",
            "version": "1.1",
            "xmlns": "http://www.topografix.com/GPX/1/1",
            "xmlns:dyo": "http://xmlns.dyomedea.com/",
            "xmlns:gpxtpx": "http://www.garmin.com/xmlschemas/TrackPointExtension/v1",
            "xmlns:gpxx": "http://www.garmin.com/xmlschemas/GpxExtensions/v3",
            "xmlns:wptx1": "http://www.garmin.com/xmlschemas/WaypointExtension/v1",
            "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
            "xsi:schemaLocation": "http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd http://www.garmin.com/xmlschemas/GpxExtensions/v3 http://www8.garmin.com/xmlschemas/GpxExtensionsv3.xsd http://www.garmin.com/xmlschemas/WaypointExtension/v1 http://www8.garmin.com/xmlschemas/WaypointExtensionv1.xsd http://www.garmin.com/xmlschemas/TrackPointExtension/v1 http://www.garmin.com/xmlschemas/TrackPointExtensionv1.xsd",
          },
          "metadata": Object {
            "time": "1970-01-01T00:00:00.000Z",
          },
        },
        "type": "gpx",
      },
      "id": "gpx/08640000000000000",
      "key": "gpx/08640000000000000",
      "value": Object {
        "rev": "1-e8125b3d924c831968288bf8786e8651",
      },
    },
    Object {
      "doc": Object {
        "_id": "gpx/08640000000000000/2trk/000000",
        "_rev": "1-a4d5a6dfa66766fd3e42c6ee328436f7",
        "doc": Object {
          "number": 0,
        },
        "type": "trk",
      },
      "id": "gpx/08640000000000000/2trk/000000",
      "key": "gpx/08640000000000000/2trk/000000",
      "value": Object {
        "rev": "1-a4d5a6dfa66766fd3e42c6ee328436f7",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });
  test('returns three rows after a gpx and a track segment have been inserted.', async () => {
    await putNewTrkseg();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/08640000000000000",
      "key": "gpx/08640000000000000",
      "value": Object {
        "rev": "1-e8125b3d924c831968288bf8786e8651",
      },
    },
    Object {
      "id": "gpx/08640000000000000/2trk/000000",
      "key": "gpx/08640000000000000/2trk/000000",
      "value": Object {
        "rev": "1-a4d5a6dfa66766fd3e42c6ee328436f7",
      },
    },
    Object {
      "id": "gpx/08640000000000000/2trk/000000/000000",
      "key": "gpx/08640000000000000/2trk/000000/000000",
      "value": Object {
        "rev": "1-9d45ff67006abf9bd493c0e38e3b9d2d",
      },
    },
  ],
  "total_rows": 6,
}
`);
  });
  test('returns four rows after a gpx and a track point have been inserted.', async () => {
    await putNewTrkpt();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/08640000000000000",
      "key": "gpx/08640000000000000",
      "value": Object {
        "rev": "1-e8125b3d924c831968288bf8786e8651",
      },
    },
    Object {
      "id": "gpx/08640000000000000/2trk/000000",
      "key": "gpx/08640000000000000/2trk/000000",
      "value": Object {
        "rev": "1-a4d5a6dfa66766fd3e42c6ee328436f7",
      },
    },
    Object {
      "id": "gpx/08640000000000000/2trk/000000/000000",
      "key": "gpx/08640000000000000/2trk/000000/000000",
      "value": Object {
        "rev": "1-9d45ff67006abf9bd493c0e38e3b9d2d",
      },
    },
    Object {
      "id": "gpx/08640000000000000/2trk/000000/000000/000000",
      "key": "gpx/08640000000000000/2trk/000000/000000/000000",
      "value": Object {
        "rev": "1-8ed7435552a904ab50c836156fdde2b4",
      },
    },
  ],
  "total_rows": 7,
}
`);
  });

  test('returns two rows after a gpx and a waypoint have been inserted.', async () => {
    await putNewWpt();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/08640000000000000",
      "key": "gpx/08640000000000000",
      "value": Object {
        "rev": "1-e8125b3d924c831968288bf8786e8651",
      },
    },
    Object {
      "id": "gpx/08640000000000000/0wpt/000000",
      "key": "gpx/08640000000000000/0wpt/000000",
      "value": Object {
        "rev": "1-5641f46ee0e0bb75493540be43e6d3b6",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });

  test('returns two rows after a gpx and a route have been inserted.', async () => {
    await putNewRte();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/08640000000000000",
      "key": "gpx/08640000000000000",
      "value": Object {
        "rev": "1-e8125b3d924c831968288bf8786e8651",
      },
    },
    Object {
      "id": "gpx/08640000000000000/1rte/000000",
      "key": "gpx/08640000000000000/1rte/000000",
      "value": Object {
        "rev": "1-1ce2ce3ed4fdc8cd652625325fa4249e",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });

  test('returns three rows after a gpx and a route point have been inserted.', async () => {
    await putNewRtept();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/08640000000000000",
      "key": "gpx/08640000000000000",
      "value": Object {
        "rev": "1-e8125b3d924c831968288bf8786e8651",
      },
    },
    Object {
      "id": "gpx/08640000000000000/1rte/000000",
      "key": "gpx/08640000000000000/1rte/000000",
      "value": Object {
        "rev": "1-1ce2ce3ed4fdc8cd652625325fa4249e",
      },
    },
    Object {
      "id": "gpx/08640000000000000/1rte/000000/000000",
      "key": "gpx/08640000000000000/1rte/000000/000000",
      "value": Object {
        "rev": "1-89cbde37e6c4a85f43e2b68c881183c0",
      },
    },
  ],
  "total_rows": 6,
}
`);
  });
});
